"""
Analytics service integration for Django projects
"""

__author__ = "Joost Cassee"
__email__ = "joost@cassee.net"
__version__ = "3.1.0"
__copyright__ = "Copyright (C) 2011-2022 Joost Cassee and contributors"
